# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function,
                        with_statement, unicode_literals)
import re
import signal
import sys
import time
import subprocess
from random import random, randint
from pyrrd.rrd import DataSource, RRA, RRD

def usage():
    print('%s rrdfile pid, ...' % __file__)
    print('sample the %cpu and %mem for specified pids, aggregated')


def sigint_handler(signal, frame):
    print("Sampling finishes: %.2f." % time.time())
    sys.exit(0)


def main():
    rrd_file = sys.argv[1]
    pids = sys.argv[2:]
    dss = [
        DataSource(dsName='cpu', dsType='GAUGE', heartbeat=4),
        DataSource(dsName='mem', dsType='GAUGE', heartbeat=4)
    ]
    rras = [RRA(cf='AVERAGE', xff=0.5, steps=1, rows=100)]
    rrd = RRD(rrd_file, ds=dss, rra=rras, step=1)
    rrd.create()
    signal.signal(signal.SIGINT, sigint_handler)

    pattern = re.compile('\s+')
    command = '/bin/ps --no-headers -o pcpu,pmem -p %s' % ' '.join(pids)
    print("Executing command: '%s'." % command)

    print("Sampling start: %d, press Ctrl-C to stop sampling ..." % int(time.time()))
    while True:
        ps = subprocess.check_output(command, shell=True)
        pcpu = 0.0
        pmem = 0.0
        for line in ps.split('\n'):
            if line.strip():
                cpu, mem = map(float, pattern.split(line.strip()))
                pcpu += cpu
                pmem += mem
        rrd.bufferValue(time.time(), pcpu, pmem)
        rrd.update()
        time.sleep(1)

    # fetch:
    # rrdtool fetch system.rrd AVERAGE --end=now --start=now-50s

    # graph:
    # rrdtool graph /vagrant/xx.png --start 1401919870 --end 1401919879 DEF:cpu=system.rrd:cpu:AVERAGE LINE2:cpu#FF0000 DEF:mem=system.rrd:mem:AVERAGE LINE:mem#ccff00

if __name__ == "__main__":
    if len(sys.argv) < 3:
        usage()
        sys.exit()

    main()
